"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _core = require("@babel/core");

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _debug = _interopRequireDefault(require("debug"));

var _fsCache = _interopRequireDefault(require("./fs-cache"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

let debug = (0, _debug.default)('WXA:BABEL_Loader');

let pkg = require('../package.json');

function readFile(p) {
  p = typeof p === 'object' ? _path.default.join(p.dir, p.base) : p;
  return _fs.default.readFileSync(p, 'utf-8');
}

function amazingCache(params, needCache) {
  let defaultOpts = {
    directory: true,
    identifier: JSON.stringify({
      '@wxa/compiler-babel': pkg.version,
      'env': process.env.NODE_ENV || 'development'
    })
  };
  let cacheParams = Object.assign({}, defaultOpts, params);

  if (needCache) {
    return (0, _fsCache.default)(cacheParams);
  } else {
    let {
      source,
      transform,
      options
    } = cacheParams;
    return transform(source, options);
  }
}

class BabelLoader {
  constructor(cwd, configs) {
    // default match file path.
    this.test = /\.js$|\.wxs$/;
    this.current = cwd; // get configuration from .babelrc, package.json or wxa.config.js
    // find .babelrc first then babel.config.js

    let babelrc = _path.default.join(this.current, '.babelrc');

    let babeljs = _path.default.join(this.current, 'babel.config.js');

    let pkg = _path.default.join(this.current, 'package.json');

    if (_fs.default.existsSync(babelrc)) {
      this.configs = JSON.parse(_fs.default.readFileSync(babelrc, 'utf-8'));
    } else if (_fs.default.existsSync(babeljs)) {
      this.configs = require(babeljs);
    } else if (_fs.default.existsSync(pkg)) {
      this.configs = require(pkg).babel;
    } // setup default babel config


    this.configs = this.configs || configs || {}; // process ignore to compat babel6

    if (typeof this.configs.ignore === 'string' || this.configs.ignore instanceof RegExp) {
      this.configs.ignore = [this.configs.ignore];
    } else if (this.configs.ignore && !Array.isArray(this.configs.ignore)) {
      throw new Error(`babel 配置 ignore 必须为Array类型`);
    } else {
      this.configs.ignore = this.configs.ignore || ["node_modules"];
    }
  }

  transform(type, ...args) {
    let map = new Map([['transform', _core.transform], ['transformFile', _core.transformFile]]);
    return new Promise((resolve, reject) => {
      debug('arguments %O', args);
      map.get(type).call(null, ...args, function (err, result) {
        if (err) reject(err);
        resolve(result);
      });
    });
  }

  async parse(mdl, cmdOptions) {
    debug('transform started %O', mdl);
    let {
      meta: {
        source: src
      },
      content: code
    } = mdl;
    let configs = this.configs;
    let type = 'transform';

    let opath = _path.default.parse(src);

    if (code == null) {
      type = 'transformFile';
    }

    if (this.checkIgnore(opath, configs.ignore) || code === '') {
      return Promise.resolve({
        code
      });
    } else {
      let ret = await amazingCache({
        source: code || readFile(src),
        options: _objectSpread({}, configs, {
          filename: src
        }),
        transform: () => {
          return this.transform(type, code || src, _objectSpread({}, configs, {
            filename: src
          }));
        }
      }, cmdOptions.cache);
      debug('transform succ %s', ret.code);
      return Promise.resolve({
        ret,
        code: ret.code,
        sourceMap: ret.map
      });
    }
  }

  checkIgnore(opath, ignore) {
    if (ignore == null) return false;
    let filepath = this.normal(opath.dir + _path.default.sep + opath.base);
    return ignore.some(str => {
      let reg = typeof str === 'object' ? str : new RegExp(str);
      return reg.test(filepath);
    });
  }

  normal(path) {
    return path.replace(/\\/g, '/');
  }

}

var _default = BabelLoader;
exports.default = _default;